;; -*- lexical-binding: t; -*-

(setq eldoc-documentation-strategy 'eldoc-documentation-compose)
(global-set-key (kbd "C-c h") 'eldoc-doc-buffer)

(defvar eldoc-frame-frame nil)
(defvar eldoc-frame-max-height 5)

(defun eldoc-frame-hide (&optional frame)
  "Hide FRAME and select it's parent frame."
  (make-frame-invisible frame)
  (select-frame-set-input-focus (frame-parameter frame 'parent-frame)))

(defun eldoc-frame-display (buffer alist)
  "Display eldoc BUFFER in child frame.
ALIST is passed to `window--display-buffer'."
  (setq mode-line-format nil)
  (let ((frame (selected-frame)))
    (if (frame-live-p eldoc-frame-frame)
        (modify-frame-parameters eldoc-frame-frame
                                 `((parent-frame . ,frame)))
      (setq eldoc-frame-frame
            (let ((after-make-frame-functions nil))
              (make-frame `((tool-bar-lines . 0)
                            (visibility)
                            (auto-hide-function . eldoc-frame-hide)
                            (minibuffer)
                            (undecorated . t)
                            (background-color . "#e5e5e5")
                            (vertical-scroll-bars)
                            (horizontal-scroll-bars)
                            (internal-border-width . 2)
                            (parent-frame . ,frame)))))
      (set-face-background 'fringe nil eldoc-frame-frame)
      (set-face-background 'internal-border "#c0c0c0" eldoc-frame-frame))
    (let ((eldoc-window (frame-selected-window eldoc-frame-frame))
          (pos (pos-visible-in-window-p (window-point) nil t)))
      (modify-frame-parameters
       eldoc-frame-frame
       `((left . ,(+ (car pos)
                     (car (window-edges nil t nil t))
                     (- (car (window-edges eldoc-window t nil t)))))
         (top . ,(+ (cadr pos)
                    (cadr (window-edges nil nil nil t))
                    (frame-char-height frame)))))
      ;; (make-frame-visible eldoc-frame-frame)
      (prog1 (window--display-buffer buffer eldoc-window 'frame alist)
        (set-window-dedicated-p eldoc-window 'soft)
        (fit-frame-to-buffer eldoc-frame-frame eldoc-frame-max-height)
        (make-frame-visible eldoc-frame-frame)))))


(add-to-list 'display-buffer-alist '("\\*eldoc.*" eldoc-frame-display))
